@@ -36,10 +36,15 @@ int main(int argc, char **argv)
 #endif
 
 	printf("%s %s\n", PACKAGE_NAME, PACKAGE_VERSION);
-	printf("Copyright 2018-2023 Michael Speck\n");
+	printf("Copyright 2018-2024 Michael Speck\n");
 	printf("Published under GNU GPL\n");
 	printf("---\n");
 
+#ifdef WITH_BUG_REPORT
+	printf("Bug reports enabled\n");
+	printf("---\n");
+#endif
+
 	srand(time(NULL));
 
 	Config config;
